//+------------------------------------------------------------------+ 
//|                                       MasterMartingale v2.00.mq4 |
//|                                               copyright 2006 DGC |
//+------------------------------------------------------------------+

//v1.00: Original version
//v1.01: Added Target and BailOut lines
//v1.02: Fixed multiple trade bug; integrated DisplayInfo routine
//v1.03: Corrected spread calculation error
//v1.04: Additional formatting of displayed objects and text
//v2.00: Add multi-trade capability; improve graphics slightly
//       Allow strategy-specific Magic Numbers; add adjustable scaling

#property copyright "Copyright  2006 DGC"

extern int     TP          = 10;
extern int     SL          = 10;
extern int     Direction   = 1;
extern double  Lots        = 0.01;
extern double  Scaler      = 2.0;
extern int     mm          = 0;
extern int     Strategy    = 1; // 1 = counter, 2 = alternator, 3 = indicator
extern int     TradeMode   = 1; // 1 = single trade, 2 = multiple trades
extern int     MN_Counter  = 962245;
extern int     MN_Altern   = 278494;
extern int     MN_Indic    = 134524;

int            LONG        = 1;     int      SINGLE      = 1;
int            SHORT       = -1;    int      MULTI       = 2;
double         dTarget     = 0;
double         dBailOut    = 0;
int            iTradeDir   = 0;
int            iLevel      = 0;
int            cnt         = 0;
int            slippage    = 3;
int            ticket      = 0;
double         lotsi       = 0;
double         mylotsi     = 0;
bool           bNeedTrade  = TRUE;
bool           bInProg     = FALSE;
string         CurrentBuild= "v2.00";
int            MagicNumber;

int init(){return(0);}
int deinit(){return(0);}

int start()
{
   int cnt = 0;

//1. Money management & trade sizing
   if (mm != 0)
      lotsi = MathRound(AccountBalance() * 0.0002) / 100;
   else
      lotsi = Lots;
   mylotsi = NormalizeDouble(lotsi * MathPow(Scaler, iLevel), 2);
   if (mylotsi > 100) mylotsi = 100;

   if (bNeedTrade == TRUE)
   {
      switch(Strategy)
      {
         case 1:  // Counter-trading mode
            iTradeDir = Direction;
            MagicNumber = MN_Counter;
            break;
            
         case 2:  // Alternating trading mode
            MagicNumber = MN_Altern;
            if (bInProg == FALSE)
               iTradeDir = Direction;
            else
               iTradeDir = -iTradeDir;            
            break;
            
         case 3:  // Indicator trading mode
            MagicNumber = MN_Indic;
            if(iCustom(NULL, 15, "i_Trend_s", 0, 0, 0, 30, 2, 20, 300, 0, 0) > 0)
               iTradeDir = LONG;
            else
               iTradeDir = SHORT;   
            break;
      }
      if (iTradeDir == LONG)
      {
         dTarget = Ask + TP * Point;
         dBailOut = Ask - SL * Point;
         OrderSend(Symbol(), OP_BUY, mylotsi, Ask, slippage, 0, 0, NULL, MagicNumber, 0, White);
         bNeedTrade = FALSE;
         bInProg = TRUE;
         ObjectCreate("Target", OBJ_HLINE, 0, 0, dTarget);
         ObjectSet("Target", OBJPROP_COLOR, Blue);
         ObjectSet("Target", OBJPROP_STYLE, STYLE_DOT);
         ObjectCreate("BailOut", OBJ_HLINE, 0, 0, dBailOut);
         ObjectSet("BailOut", OBJPROP_COLOR, Red);
         ObjectSet("BailOut", OBJPROP_STYLE, STYLE_DOT);
      }
      else
      {
         dTarget = Bid - TP * Point;
         dBailOut = Bid + TP * Point;
         OrderSend(Symbol(), OP_SELL, mylotsi, Bid, slippage, 0, 0, NULL, MagicNumber, 0, Red);
         bNeedTrade = FALSE;
         bInProg = TRUE;
         ObjectCreate("Target", OBJ_HLINE, 0, 0, dTarget);
         ObjectSet("Target", OBJPROP_COLOR, Blue);
         ObjectSet("Target", OBJPROP_STYLE, STYLE_DOT);
         ObjectCreate("BailOut", OBJ_HLINE, 0, 0, dBailOut);
         ObjectSet("BailOut", OBJPROP_COLOR, Red);
         ObjectSet("BailOut", OBJPROP_STYLE, STYLE_DOT);
      }
   }
   
   if (iTradeDir == LONG)
   {
      if (Bid <= dBailOut)
      {
         if (TradeMode == SINGLE) CloseAll();
         bNeedTrade = TRUE;
         ObjectDelete("Target");
         ObjectDelete("BailOut");
         iLevel ++;
      }
      if (Bid >= dTarget)
      {
         CloseAll();
         bNeedTrade = TRUE;
         ObjectDelete("Target");
         ObjectDelete("BailOut");
         iLevel = 0;
         bInProg = FALSE;
      }   
   }
   
   if (iTradeDir == SHORT)
   {
      if (Ask >= dBailOut)
      {
         if (TradeMode == SINGLE) CloseAll();
         bNeedTrade = TRUE;
         ObjectDelete("Target");
         ObjectDelete("BailOut");
         iLevel ++;
      }
      if (Ask <= dTarget)
      {
         CloseAll();
         bNeedTrade = TRUE;
         ObjectDelete("Target");
         ObjectDelete("BailOut");
         iLevel = 0;
         bInProg = FALSE;
      }      
   }
   Comment("Strategy = ", Strategy,
           "  TradeMode = ", TradeMode,
           "  Direction = ", iTradeDir,
           "\nLots = ", mylotsi,
           "  Level = ", iLevel,
           "  Build: ", CurrentBuild);
   return(0);
}

int CloseAll()
{
   for (cnt = OrdersTotal() - 1; cnt >= 0; cnt --)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      if (OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber) 
         OrderClose(OrderTicket(), OrderLots(), OrderClosePrice(), slippage, Blue);
   }
   return(0);
}